/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#include "fsarchiver.h"

#ifdef OPTION_CRYPTO_SUPPORT

#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "common.h"
#include "crypto.h"

int crypt_block_blowfish(u64 insize, u64 *outsize, u8 *inbuf, u8 *outbuf, u8 *password, int passlen, int enc)
{
	EVP_CIPHER_CTX ctx;
        u8 iv[] = "fsarchiv";
	int cursize=0;
	int outlen=0;
	
	// init cipher structures
        EVP_CIPHER_CTX_init(&ctx);
        EVP_CipherInit_ex(&ctx, EVP_bf_cfb(), NULL, NULL, NULL, enc);
        EVP_CIPHER_CTX_set_key_length(&ctx, passlen);
        EVP_CipherInit_ex(&ctx, NULL, NULL, password, iv, enc);
	
	if(EVP_CipherUpdate(&ctx, outbuf+cursize, &outlen, inbuf, insize)!=1)
	{	errprintf("EVP_CipherUpdate() failed\n");
		EVP_CIPHER_CTX_cleanup(&ctx);
		return 1;
	}
	cursize+=outlen;
	if(EVP_CipherFinal_ex(&ctx, outbuf+cursize, &outlen)!=1)
	{	errprintf("EVP_CipherFinal_ex() failed\n");
		EVP_CIPHER_CTX_cleanup(&ctx);
		return 1;
	}
	cursize+=outlen;
	
	EVP_CIPHER_CTX_cleanup(&ctx);
	
	*outsize=cursize;
	
	return 0;
}

#endif // OPTION_CRYPTO_SUPPORT

int generate_random_buffer(u8 *buf, int bufsize)
{
	int ret;
	
	memset(buf, 0, bufsize);
#ifdef OPTION_CRYPTO_SUPPORT
	ret=RAND_pseudo_bytes(buf, bufsize);
#else // OPTION_CRYPTO_SUPPORT
	int i;
	for (i=0; i < bufsize; i++)
		buf[i]=rand(void)%256;
#endif // OPTION_CRYPTO_SUPPORT
	return 0;
}
