/*
 * fsarchiver: Filesystem Archiver
 *
 * Copyright (C) 2008-2009 Francois Dupoux.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License v2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Homepage: http://www.fsarchiver.org
 */

#ifndef __DICO_H__
#define __DICO_H__

#include "types.h"

enum	{DICO_ESUCCESS=0, DICO_ENOENT, DICO_EINVAL, DICO_EBADSIZE, DICO_EFULL, DICO_EMEM, DICO_EDUPLICATE, DICO_EINVALCHAR};
enum	{DICTYPE_NULL=0, DICTYPE_U8, DICTYPE_U16, DICTYPE_U32, DICTYPE_U64, DICTYPE_DATA, DICTYPE_STRING};

struct s_dico { struct s_dicoitem *head; };

struct s_dicoitem
{
	u8			type;
	u8			section;
	u16			key;
	u16			size;
	char			*data;
	struct s_dicoitem	*next;
};

struct s_dico *dico_alloc(); // dynamic memory allocation
int dico_destroy(struct s_dico *d);
int dico_show(struct s_dico *d, u8 section, char *debugtxt);
int dico_count_all_sections(struct s_dico *d);
int dico_count_one_section(struct s_dico *d, u8 section);
int dico_add_data(struct s_dico *d, u8 section, u16 key, const void *data, u16 size);
int dico_add_generic(struct s_dico *d, u8 section, u16 key, const void *data, u16 size, u8 type);
int dico_get_generic(struct s_dico *d, u8 section, u16 key, void *data, u16 maxsize, u16 *size);
int dico_get_data(struct s_dico *d, u8 section, u16 key, void *data, u16 maxsize, u16 *size);
int dico_add_u16(struct s_dico *d, u8 section, u16 key, u16 data);
int dico_add_u32(struct s_dico *d, u8 section, u16 key, u32 data);
int dico_add_u64(struct s_dico *d, u8 section, u16 key, u64 data);
int dico_get_u16(struct s_dico *d, u8 section, u16 key, u16 *data);
int dico_get_u32(struct s_dico *d, u8 section, u16 key, u32 *data);
int dico_get_u64(struct s_dico *d, u8 section, u16 key, u64 *data);
int dico_add_string(struct s_dico *d, u8 section, u16 key, const char *szstring);
int dico_get_string(struct s_dico *d, u8 section, u16 key, char *buffer, u16 bufsize);

#endif // __DICO_H__
